/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.jbi.management.task.installation.install.sl;

import java.io.File;

import org.ow2.petals.PetalsException;
import org.ow2.petals.jbi.descriptor.JBIDescriptor;
import org.ow2.petals.jbi.descriptor.SharedLibrary;
import org.ow2.petals.jbi.management.admin.AdminServiceMBean;
import org.ow2.petals.jbi.management.task.AbstractCheckPackageTask;
import org.ow2.petals.jbi.management.task.installation.InstallationUtils;
import org.ow2.petals.jbi.management.util.PackageHandler;
import org.ow2.petals.util.LoggingUtil;
import org.ow2.petals.util.StringHelper;

/**
 * This task checks the validity of the shared lib package.
 *
 * @author ofabre - EBM Websourcing
 *
 */
public class SLCheckPackageTask extends AbstractCheckPackageTask {


    /**
     * JBI Admin Service :: JMX Agent Admin
     */
    private AdminServiceMBean adminService;

    public SLCheckPackageTask(LoggingUtil log, PackageHandler packageHandler,
            AdminServiceMBean adminService) {
        super(log, packageHandler);
        this.adminService = adminService;
    }

    /**
     * Check the validity of the shared lib archive
     *
     * @param descriptor
     *            the shared lib <code>JBIDescriptor</code>
     * @param filePackage
     *            the service assembly package root
     * @return true if checking is completly ok and false otherwise
     * @throws PetalsException
     * @throws ComponentCheckingException
     */
    @Override
    protected void checkPackage(JBIDescriptor descriptor, File filePackage)
    throws PetalsException {
        String msg = null;

        /*
         * check if jbi descriptor has a shared-library element
         */

        SharedLibrary sl = descriptor.getSharedLibrary();
        if (sl == null) {
            msg = "The shared-library production element was not found in "
                + "JBI Descriptor within installation package: "
                + filePackage.getAbsolutePath();

            log.error(msg);
            throw new PetalsException(msg);
        } else {

            /*
             * check if it is already loaded by package uuid
             */

            String slName = sl.getIdentification().getName();

            if (StringHelper.isNullOrEmpty(slName)) {
                msg = "The shared library name must be non null and non empty :"
                    + filePackage.getAbsolutePath();
                log.error(msg);
                throw new PetalsException(msg);
            }

            if (isSharedLibraryLoaded(slName)) {
                msg = "Other shared library package already exists loaded in "
                    + "the component framework: " + slName;

                log.error(msg);
                throw new PetalsException(msg);
            } else {
                /*
                 * Check the classpath element of the shared library
                 */

                InstallationUtils.checkClassPathElements(filePackage, sl
                        .getSharedLibraryClassPath());

            }

        }
    }

    private boolean isSharedLibraryLoaded(String slName) {
        return adminService.getSharedLibraryByName(slName) != null;
    }


    @Override
    protected String extractEntityName(JBIDescriptor descriptor) {
        return descriptor.getSharedLibrary().getIdentification().getName();
    }

}
