/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.messaging.routing.EndpointChooser;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;

import javax.jbi.messaging.MessageExchange;
import javax.jbi.servicedesc.ServiceEndpoint;

import org.ow2.petals.jbi.messaging.routing.RoutingException;

/**
 * @author Roland NAUDIN - EBM WebSourcing
 * 
 */
public class StandaloneStrategy implements Strategy {

    private static final int NUMBER_STRATEGY_PARAMETERS = 0;

    /**
     * Random used to elect the endpoint
     */
    private Random random = null;

    public StandaloneStrategy() {
        this.random = new Random();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.jbi.messaging.routing.EndpointChooser.Strategy#validateStrategyParameters(java.util.List)
     */
    public void validateStrategyParameters(List<String> strategyParameters) throws RoutingException {
        // no parameters
        if (strategyParameters.size() != NUMBER_STRATEGY_PARAMETERS) {
            throw new RoutingException("No parameters required by the standalone routing strategy");
        }

    }

    /**
     * The standalone strategy. The endpoint is elected according to a random
     * election.
     * 
     * @see org.ow2.petals.jbi.messaging.routing.EndpointChooser.Strategy#electEndpoint(java.util.List,
     *      javax.jbi.messaging.MessageExchange)
     */
    public ServiceEndpoint electEndpoint(List<ServiceEndpoint> endpoints, MessageExchange exchange)
            throws RoutingException {
        Map<Float, ServiceEndpoint> weightingEndpoints = new HashMap<Float, ServiceEndpoint>();
        ServiceEndpoint result = null;
        Float maxValue = new Float(0);

        if (endpoints != null && endpoints.size() > 0) {
            for (ServiceEndpoint endpoint : endpoints) {
                weightingEndpoints.put(new Float(random.nextFloat()), endpoint);
            }

            for (Float weighting : weightingEndpoints.keySet()) {
                if (weighting.compareTo(maxValue) >= 0) {
                    result = weightingEndpoints.get(weighting);
                    maxValue = weighting;
                }
            }
        }

        return result;
    }

}
