/**
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): pierre.garcia@inrialpes.fr
 *
 */

package org.ow2.petals.jbi.messaging.transport.platform;

import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.ow2.petals.jbi.messaging.exchange.MessageExchangeImpl;

/**
 * Interface used by the Transport Protocol implementers to notify the
 * dispatcher that messages are available. When the message received is
 * standard, the {@link TransportProtocolListener#availableExchangesMap} is called.
 * When it is the result of a synchronous response , the
 * {@link TransportProtocolListener#syncMessageReceived} is called.
 * 
 * @author garciap
 */
@Interface(name = TransportProtocolListener.SERVER_ITF)
public interface TransportProtocolListener {

    /**
     * Name of the interface
     */
    public static final String SERVER_ITF = "dispatcher";

    /**
     * Notify the dispatcher that the transporter specified has a new message
     * available for the component identified by its name.
     * 
     * @param transportProtocol
     *            the Transport Protocol instance
     * @param componentName
     *            the name of the component involved
     */
    public void messageAvailable(TransportProtocol transportProtocol,
        String componentName);

    /**
     * Notify the dispatcher that a synchronized response has been received.
     * 
     * @param messageExchange
     *            the synchronized response
     * 
     */
    public void syncMessageReceived(MessageExchangeImpl messageExchange);
}
