/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.jbi.messaging.transport.platform.joram;

import javax.jbi.JBIException;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.ObjectMessage;

import org.ow2.petals.jbi.messaging.exchange.MessageExchangeImpl;
import org.ow2.petals.jbi.messaging.transport.platform.TransportProtocolListener;
import org.ow2.petals.util.LoggingUtil;

/**
 * Implementation of the JMS message listener which handles sync exchanges. When
 * a JMS message is received, the message is unserialized into a JBI exchange,
 * then the exchange is sent to the dispatcher.
 * 
 * @author rnaudin
 * 
 */
public class SyncExchangeMessageListener implements MessageListener {

    /**
     * The logging util
     */
    private final LoggingUtil log;

    /**
     * The dispatcher instance
     */
    private final TransportProtocolListener dispatcher;

    /**
     * The name of the related component
     */
    private final String componentName;

    /**
     * A serializer instance
     */
    private final ObjectSerializer serializer = new ObjectSerializer();

    /**
     * The counter of the recept in processing
     */
    private int receptCounter;

    /**
     * Creates a new instance of {@link SyncExchangeMessageListener}
     * 
     * @param log
     * @param dispatcher
     */
    public SyncExchangeMessageListener(final LoggingUtil log,
        final TransportProtocolListener dispatcher, String componentName) {
        this.log = log;
        this.dispatcher = dispatcher;
        this.componentName = componentName;
        receptCounter = 0;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jms.MessageListener#onMessage(javax.jms.Message)
     */
    public void onMessage(Message jmsMessage) {
        log.start();
        MessageExchangeImpl exchange = null;

        receptCounter++;

        if (jmsMessage instanceof ObjectMessage) {
            try {
                exchange = serializer.jms2jbi(jmsMessage);
            } catch (JBIException e) {
                log
                    .error("Cannot unserialize a retrieved synchronious response message for the component '"
                        + componentName + "'");
            }
        } else {
            log
                .error("The received JMS message is not an ObjectMessage instance");
        }

        dispatcher.syncMessageReceived(exchange);

        receptCounter--;

        log.end();
    }

    /**
     * Wait that all the ongoing recept threads awake the matched issuer thread
     * 
     */
    public void terminate() {
        log.start();
        int forceTerminate = 0;

        while (receptCounter > 0 && forceTerminate < 10) {
            try {
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                forceTerminate = 11;
            }
            forceTerminate++;
        }
        
        if (forceTerminate > 10) {
            log.debug("Force the stop of the synchronous transport device");
        }
        
        log.end();
    }
}
