/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.monitoring.system;

import java.lang.management.MemoryUsage;

/**
 * SystemMonitoring interface provides system monitoring methods for the JVM
 * that handle the current Petals container. Monitoring methods provide
 * information about memory usage, threads and classes
 * 
 * @author ofabre - eBM Websourcing
 * 
 */
public interface SystemMonitoringMBean {

    /**
     * @see java.lang.management.MemoryMXBean#getHeapMemoryUsage()
     * @return
     */
    MemoryUsage getMemoryUsage();

    /**
     * @see java.lang.management.ThreadMXBean#getPeakThreadCount()
     * @return
     */
    int getPeakThreadCount();

    /**
     * @see java.lang.management.ThreadMXBean#getDaemonThreadCount()
     * @return
     */
    int getDaemonThreadCount();

    /**
     * @see java.lang.management.ThreadMXBean#getThreadCount()
     * @return
     */
    int getThreadCount();

    /**
     * @see java.lang.management.ThreadMXBean#getTotalStartedThreadCount()
     * @return
     */
    long getTotalStartedThreadCount();

    /**
     * @see java.lang.management.OperatingSystemMXBean#getAvailableProcessors()
     * @return
     */
    int getAvailableProcessors();

    /**
     * @see java.lang.management.RuntimeMXBean#getUptime()
     * @return
     */
    long getUptime();

    /**
     * @see java.lang.management.ClassLoadingMXBean#getLoadedClassCount()
     * @return
     */
    int getLoadedClassCount();

    /**
     * @see java.lang.management.ClassLoadingMXBean#getTotalLoadedClassCount()
     * @return
     */
    long getTotalLoadedClassCount();

    /**
     * @see java.lang.management.ClassLoadingMXBean#getUnloadedClassCount()
     * @return
     */
    long getUnloadedClassCount();

}
