/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: Loaders.java 6254 2007-12-13 10:49:31Z rnaudin $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.platform.classloader;

import java.util.HashMap;
import java.util.Map;

/**
 * This class gathers the class loaders used in Petals.
 *
 * @version $Revision: 6254 $ $Date: 2006-05-24 12:42:07 +0200 (mer., 24 mai
 *          2006) $
 * @since Petals 1.0
 * @author <a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 * @author ddesjardins - eBMWebsourcing
 */
public final class Loaders {

    /**
     * the common lodaer name
     */
    public static final String COMMON_LOADER = "common";

    /**
     * the loader.
     */
    private static Loaders singletonInstance;

    /**
     * the shared library class loader.
     */
    private static SharedLibrariesClassLoader slClassLoader;

    /**
     * the class loaders table.
     */
    private Map<String, PetalsClassLoader> classLoadersTable =
        new HashMap<String, PetalsClassLoader>();

    /**
     * Constructor. An unique SharedLibrariesClassLoader is created. It must be
     * used to reference all classloader created for shared libraries.
     *
     */
    private Loaders() {
        super();
        Loaders.slClassLoader = new SharedLibrariesClassLoader();
    }

    /**
     * get instance of the loaders.
     * @return
     *      the loaders
     */
    public static Loaders getInstance() {
        if (Loaders.singletonInstance == null) {
            Loaders.singletonInstance = new Loaders();
        }
        return Loaders.singletonInstance;
    }

    /**
     *  Check if the library is inside the loaders.
     * @param loaderName
     *      the loader name
     * @return
     *       true if exist
     */
    public boolean contains(final String loaderName) {
        return this.classLoadersTable.containsKey(loaderName);
    }

    /**
     *  Get the petals class loader.
     * @param loaderName
     *           the loader name
     * @return
     *           the petals class loader
     */
    public PetalsClassLoader get(final String loaderName) {
        return this.classLoadersTable.get(loaderName);
    }

    /**
     * get the Shared Libraries ClassLoader.
     * @return
     *     the Shared Libraries ClassLoader
     */
    public SharedLibrariesClassLoader getSharedLibrariesClassLoader() {
        return Loaders.slClassLoader;
    }

    /**
     *
     * @param loaderName
     * @param cl
     * @return
     */
    public ClassLoader put(final String loaderName, final PetalsClassLoader cl) {
        return this.classLoadersTable.put(loaderName, cl);
    }

    public PetalsClassLoader remove(String loaderName) {
        return this.classLoadersTable.remove(loaderName);
    }
}
